Unit Itabcalp;
{copyright 1995,1996,1997 by Stellberg Computer GmbH}
(*********************************************
This form unit is used by the DateEdit component.
*********************************************)
Interface

Uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Buttons, StdCtrls;

Const
   m_DaysPerMonth: Array[1..12] of Integer = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
   m_DayTitles : Array[0..6] of String[2] = ('So','Mo','Di','Mi','Do','Fr','Sa');
   BORDER = 2;
   TEXT_INDENT = 2;
   BUTTON_WIDTH = 16;

Type
  {////  Calendar Form Type Definition /////}
  TItabCalPop = Class( TForm )
    {SpeedButton1: TSpeedButton;}
    Procedure FormCancel;
    Procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    Procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    Procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    Procedure FormKeyDown(Sender: TObject; Var Key: Word;
      Shift: TShiftState);
    Procedure FormPaint(Sender: TObject);
    Procedure FormKeyPress(Sender: TObject; Var Key: Char);
  Private
    m_CurrentDateSelected: TDateTime;
    m_FontWidth : Integer;
    m_FontHeight : Integer;
    m_DateArray : Array[1..42] of String[2];
    m_CurrentDateIndex : Integer;
    m_PreviousDateIndex : Integer;
    m_PreviousDateRect : TRect;
    m_MouseDown : BOOL;
    m_CurrentDay, m_CurrentYear, m_CurrentMonth : Word;
    m_PreviousDay, m_PreviousYear, m_PreviousMonth : Word;
    ctlParent: TComponent;
  Protected
    Function DaysInMonth(nMonth : Integer): Integer;
    Procedure DrawButtons;
    Procedure DrawCalendarBorder;
    Procedure DrawDates;
    Procedure DrawDaysHeader;
    Procedure DrawFocusFrame(nIndex : Integer);
    Procedure DrawMonthHeader;
    Function GetMonthBegin: Integer;
    Function GetCalendarRect : TRect;
    Function GetLeftButtonRect : TRect;
    Function GetRightButtonRect : TRect;
    Function GetRectFromIndex(nIndex : Integer): TRect;
    Function GetIndexFromDate : Integer;
    Function GetIndexFromPoint(nLeft : Integer ; nTop : Integer) : Integer;
    Function IsLeapYear: Boolean;
    Procedure LoadDateArray ;
    Procedure NextDay;
    Procedure PrevDay;
    Procedure NextWeek;
    Procedure PrevWeek;
    Procedure NextMonth;
    Procedure PrevMonth;
    Procedure NextYear;
    Procedure PrevYear;
    Procedure SetDate(nDays : Integer);
  Public
    Constructor Create( AOwner: TComponent ); Override;
  End;

Var
  ItabCalPop: TItabCalPop;

Implementation

{$R *.DFM}

Uses
  ITabEdit;

Function PointInRect( Const rectTest: TRect; X, Y: Integer ): Boolean;
Begin
  Result := ( ( X >= rectTest.Left ) and ( X <= rectTest.Right ) and
     ( Y >= rectTest.Top ) and ( Y <= rectTest.Bottom ) );
End;

{************************** Create ************************
 *****  This procedure is used to initialize values   *****
 *****  for control owner, calendar position and      *****
 *****  other resources.                              *****
 **********************************************************}
Constructor TItabCalPop.Create(AOwner: TComponent);
Var
   tmTextMetrics : TTextMetric;
   editOwner: TIsamDateEdit;
   rectPlace: TRect;
   ptUpper, ptLower: TPoint;
Begin
  Inherited Create(AOwner);

  {If the FontWidth is not set, determine Font Height and Width for positioning Dates}
  With Canvas do
     Begin
        Font.Name := 'MS Sans Serif';
        Font.Size := 6;
        Pen.Color := clBlack;
        GetTextMetrics(Handle, tmTextMetrics);
        m_FontWidth :=  Round(tmTextMetrics.tmAveCharWidth + tmTextMetrics.tmAveCharWidth * 6 / 10);
        m_FontHeight :=  Round(tmTextMetrics.tmHeight + tmTextMetrics.tmHeight / 3);
     End;

  {Initialize form Height & Width based on Font }
  Height := (m_FontHeight * 6) + (m_FontHeight * 2) + BORDER;
  Width := ((m_FontWidth *3) * 7) + (2* BORDER) + (2* TEXT_INDENT);

{ Dynamically set the size and position }
  editOwner := TIsamDateEdit( AOwner );
  ctlParent := editOwner;
  rectPlace := editOwner.ClientRect;
  ptUpper.X := rectPlace.Left;
  ptUpper.Y := rectPlace.Top;
  ptUpper := editOwner.ClientToScreen( ptUpper );
  ptLower.X := rectPlace.Right;
  ptLower.Y := rectPlace.Bottom;
  ptLower := editOwner.ClientToScreen( ptLower );

  { If too far down, pop the calendar above the control }
  if ptUpper.X + 1 + Width > Screen.Width then
     Left := Screen.Width - Width - 1
  Else
     Left := ptUpper.X + 1;
  if ptLower.Y + 1 + Height > Screen.Height then
     Top := ptUpper.Y - Height
  Else
     Top := ptLower.Y + 1;

  { define initial date }
  if TIsamDateEdit( ctlParent ).Text <> '' then
     m_CurrentDateSelected := StrToDate( TIsamDateEdit( ctlParent ).Text )
  Else
     m_CurrentDateSelected := Date;

  {Extract date Components}
  DecodeDate( m_CurrentDateSelected, m_CurrentYear, m_CurrentMonth, m_CurrentDay );
  m_CurrentDateIndex := m_CurrentDay + GetMonthBegin - 1;
  m_PreviousDateIndex := 0;

  LoadDateArray;
  m_MouseDown := False;
End;


{********************** Days In Month *********************
 *****  This function returns the number of days in   *****
 *****  the month specified in nMonth.                *****
 **********************************************************}
Function TItabCalPop.DaysInMonth(nMonth : Integer): Integer;
Begin
  Result := m_DaysPerMonth[nMonth];
  if ( nMonth = 2 ) and IsLeapYear then Inc( Result ); { leap-year Feb is special }
End;


{******************** Draw Butttons ***********************
 **********************************************************}
Procedure TItabCalPop.DrawButtons;
Var
  LeftButtonRect: TRect;
  RightButtonRect : TRect;
  OldStyle : TBrushStyle;
Begin
  With Canvas do
     Begin
        LeftButtonRect := GetLeftButtonRect;
        RightButtonRect := GetRightButtonRect;

        { Select Black Pen}
        Pen.Style := psSolid;
        Pen.Width := 1;
        Pen.Color := clBlack;

        { Draw Button Outlines }
        Rectangle( LeftButtonRect.Left, LeftButtonRect.Top, LeftButtonRect.Right, LeftButtonRect.Bottom );
        Rectangle( RightButtonRect.Left, RightButtonRect.Top, RightButtonRect.Right, RightButtonRect.Bottom );

        { Create Embossed effect - Outline left & upper in white}
        Pen.Color := clWhite;
        MoveTo( LeftButtonRect.Left + 1, LeftButtonRect.Bottom - 2 );
        LineTo( LeftButtonRect.Left + 1, LeftButtonRect.Top + 1 );
        LineTo( LeftButtonRect.Right - 2, LeftButtonRect.Top + 1 );

        MoveTo( RightButtonRect.Left + 1, RightButtonRect.Bottom - 2 );
        LineTo( RightButtonRect.Left + 1, RightButtonRect.Top + 1 );
        LineTo( RightButtonRect.Right - 2, RightButtonRect.Top + 1 );

        { Create Embossed effect - Outline right & bottom in gray }
        Pen.Color := clGray;
        MoveTo( LeftButtonRect.Right -2, LeftButtonRect.Top +  1 );
        LineTo( LeftButtonRect.Right - 2, LeftButtonRect.Bottom - 2 );
        LineTo( LeftButtonRect.Left + 1, LeftButtonRect.Bottom - 2 );

        MoveTo( RightButtonRect.Right - 2, RightButtonRect.Top + 1 );
        LineTo( RightButtonRect.Right - 2, RightButtonRect.Bottom - 2 );
        LineTo( RightButtonRect.Left + 1, RightButtonRect.Bottom - 2 );

        {Draw Arrow}
        Brush.Color := clBlack;
        OldStyle :=Brush.Style;
        Brush.Style := bsSolid;
        Polygon([Point(LeftButtonRect.Right - 5,LeftButtonRect.Top + 3),
                 Point(LeftButtonRect.Right - 5,LeftButtonRect.Bottom - 4),
                 Point(LeftButtonRect.Left + 3,LeftButtonRect.Top + 7)]);
        Polygon([Point(RightButtonRect.Left + 4,RightButtonRect.Top + 3),
                 Point(RightButtonRect.Left + 4,RightButtonRect.Bottom - 4),
                 Point(RightButtonRect.Right - 4,RightButtonRect.Top + 7)]);
        Brush.Color :=clSilver;
        Brush.Style := OldStyle;
        Pen.Color := clBlack;
     End;
End;

{*************** Draw Calendar Border *********************
 **********************************************************}
Procedure TItabCalPop.DrawCalendarBorder;
Var
  rectDraw: TRect;
Begin
  rectDraw := ClientRect;
  With Canvas do
     Begin
        { Select Black Pen to outline Window }
        Pen.Style := psSolid;
        Pen.Width := 1;
        Pen.Color := clBlack;

        { Outline the window in black }
        Rectangle( rectDraw.Left, rectDraw.Top, rectDraw.Right, rectDraw.Bottom );

        { Create Embossed effect - Outline left & upper in white}
        Pen.Color := clWhite;
        MoveTo( 0, rectDraw.Bottom - 1 );
        LineTo( 0, 0 );
        LineTo( rectDraw.Right - 1, 0 );

        { Create Embossed effect - Outline right & bottom in gray }
        Pen.Color := clGray;
        LineTo( rectDraw.Right - 1, rectDraw.Bottom - 1 );
        LineTo( 0, rectDraw.Bottom - 1 );

        { Reset Pen Color }
        Pen.Color := clBlack;
     End;
End;

{*********************** Draw Dates ***********************
 **********************************************************}
Procedure TItabCalPop.DrawDates;
Var
   nIndex, nWeek, nDay: Integer;
   pDate: PChar;
   TempRect: Trect;
   dtTest: TDateTime;
Begin
  pDate := StrAlloc( 3 );

  With Canvas do
     Begin
        { Define normal font }
        Font.Style := [];
        Pen.Color := clBlack;

        { Cycle through the weeks }
        for nWeek := 1 to 6 do
           Begin
              { Cycle through the days }
              for nDay := 1 to 7 do
                 Begin
                    nIndex := nDay + ( ( nWeek - 1 ) * 7 );
                    StrPCopy( pDate, m_DateArray[nIndex] );
                    if m_DateArray[nIndex] <> '  ' then
                       Begin
                          dtTest := EncodeDate( m_CurrentYear, m_CurrentMonth, StrToInt( m_DateArray[nIndex] ) );
                             Font.Color := clBlack;
                       End;
                    TempRect := GetCalendarRect;

                    With TempRect do
                    Begin
                       Left := Left + ((m_FontWidth * 3) * (nDay - 1));
                       Top := (m_FontHeight * nWeek ) + m_FontHeight + Border;
                       Bottom := Top +  m_FontHeight ;
                       Right := Left + m_fontWidth * 3;
                    End;

                    DrawText( Handle, pDate, Length( m_DateArray[nIndex] ),
                              TempRect, ( DT_CENTER or DT_VCENTER or DT_TOP or DT_SINGLELINE ) );

                 End;
                 nIndex := nIndex;
           End;
     End;
     StrDispose( pDate );
End;

{*********************** Draw Days ************************
 **********************************************************}
Procedure TItabCalPop.DrawDaysHeader;
Var
   i: Integer;
   pDay: PChar;
   TempRect: Trect;
Begin
  pDay := StrAlloc( 3 );

  { Calculate Rect Top.  2nd line = FontHeight * 2 }
  TempRect := ClientRect;
  TempRect.Top := m_FontHeight + BORDER;
  TempRect.Bottom := TempRect.Top + m_FontHeight;

  {Calculate each date rect. rect = FontWidth * 3 (width of two chars + space) }
  TempRect.Left := TempRect.Left + BORDER + TEXT_INDENT;
  TempRect.Right := BORDER + TEXT_INDENT + ( m_FontWidth * 3 );

  { Cycle through the days }
  for i := 0 to 6 do
     Begin
        StrPCopy( pDay, m_DayTitles[i] );
        DrawText( Canvas.Handle, pDay, 2, TempRect,
                ( DT_CENTER or DT_TOP or DT_SINGLELINE ) );
        TempRect.Left := TempRect.Right;
        TempRect.Right := TempRect.Right + m_FontWidth * 3;
     End;

     { Draw line below days }
     With Canvas do
        Begin
           TempRect.Top := TempRect.Bottom - 3;
           TempRect.Bottom := TempRect.Top + 2;
           TempRect.Left := ClientRect.Left + BORDER + TEXT_INDENT;
           TempRect.Right := BORDER + TEXT_INDENT + ( m_FontWidth * 3 * 7 );

           Pen.Color := clGray;
           MoveTo( TempRect.Left , TempRect.Top);
           LineTo( TempRect.Right, TempRect.Top );
           Pen.Color := clWhite;
           MoveTo( TempRect.Left,  TempRect.Top + 1 );
           LineTo( TempRect.Right, TempRect.Top + 1  );
        End;

     StrDispose( pDay );
End;

{******************** Draw Month Header *******************
 **********************************************************}
Procedure TItabCalPop.DrawMonthHeader;
Var
   sMonth : ShortString;
   pMonth : PChar;
   TempRect : Trect;
Begin
  pMonth := StrAlloc( 30 );
  With Canvas do
     Begin
        Font.Style := [fsBold];
        Font.Color := clBlack;
        sMonth := FormatDateTime( 'dd. mmmm yyyy', m_CurrentDateSelected );

        pMonth := StrAlloc( Length( sMonth ) + 1 );
        StrPCopy( pMonth, sMonth );

        TempRect := ClientRect;
        TempRect.Top := BORDER;
        TempRect.Left := BORDER + TEXT_INDENT + BUTTON_WIDTH;
        TempRect.Right := TempRect.Right - BORDER - TEXT_INDENT - BUTTON_WIDTH;
        TempRect.Bottom := m_FontHeight;

        Brush.Color := clSilver;
        Brush.Style := bsSolid;
        FillRect( TempRect );

        DrawText( Handle, pMonth, Length( sMonth ), TempRect,
                ( DT_CENTER or DT_VCENTER or DT_BOTTOM or DT_SINGLELINE ) );
     End;
     StrDispose( pMonth );
End;

{******************** Draw Focus Frame ********************
 **********************************************************}
Procedure TItabCalPop.DrawFocusFrame( nIndex: Integer);
Var
  pDate :PChar;
  TempRect : TRect;
  dtTest: TDateTime;
Begin
  pDate := StrAlloc( 3 );
  if ( nIndex > 0 ) and ( nIndex < 42 ) then
     if m_DateArray[nIndex] <> '  ' then
        Begin
           { Erase Previous Date Focus}
           if m_PreviousDateIndex > 0 then
              Begin
                 Canvas.Font.Style := [];
                 StrPCopy( pDate, m_DateArray[m_PreviousDateIndex] );
                 Canvas.Brush.Color := clSilver;
                 TempRect := GetRectFromIndex(m_PreviousDateIndex);
                 Canvas.FillRect(TempRect);
                 DrawText( Canvas.Handle, pDate, Length( m_DateArray[m_PreviousDateIndex] ),
                           TempRect, ( DT_CENTER or DT_VCENTER or DT_TOP or DT_SINGLELINE ) );
              End;

           {Draw the Date in Bold font}
           Canvas.Font.Style := [fsBold];
           dtTest := EncodeDate( m_CurrentYear, m_CurrentMonth, StrToInt( m_DateArray[nIndex] ) );
              Canvas.Font.Color := clBlack;
           TempRect := GetRectFromIndex(nIndex);
           StrPCopy( pDate, m_DateArray[nIndex] );
           DrawText( Canvas.Handle, pDate, Length( m_DateArray[nIndex] ),
                     TempRect, ( DT_CENTER or DT_VCENTER or DT_TOP or DT_SINGLELINE ) );

           { Frame date with Shadow }
           Canvas.Pen.Color := clGray;
           Canvas.MoveTo( TempRect.Left, TempRect.Bottom - 1 );
           Canvas.LineTo( TempRect.Left, TempRect.Top );
           Canvas.LineTo( TempRect.Right - 1, TempRect.Top );

           { Frame date with Highlight }
           Canvas.Pen.Color := clWhite;
           Canvas.LineTo( TempRect.Right - 1, TempRect.Bottom - 1 );
           Canvas.LineTo( TempRect.Left, TempRect.Bottom - 1 );

           { Restore Canvas settings}
           Canvas.Pen.Color := clBlack;
           Canvas.Font.Style := [];

        End;
  StrDispose( pDate );
End;

{********************* Form Cancel ************************
 **********************************************************}
Procedure TItabCalPop.FormCancel;
Begin
  m_MouseDown := False;
  ModalResult := -1;
End;

{******************* Form Key Down ************************
 **********************************************************}
Procedure TItabCalPop.FormKeyDown(Sender: TObject; Var Key: Word;
  Shift: TShiftState);
Begin
     Case key of
          VK_Left : Begin
                        PrevDay;
                        if (m_CurrentMonth <> m_PreviousMonth) or
                           (m_CurrentYear <> m_PreviousYear) then
                           Refresh
                        Else
                           DrawFocusFrame(m_CurrentDateIndex);
                     End;
          VK_Right : Begin
                        NextDay;

                        if (m_CurrentMonth <> m_PreviousMonth) or
                           (m_CurrentYear <> m_PreviousYear) then
                           Refresh
                        Else
                           DrawFocusFrame(m_CurrentDateIndex);
                     End;
          VK_Up : Begin
                        PrevWeek;
                        if (m_CurrentMonth <> m_PreviousMonth) or
                           (m_CurrentYear <> m_PreviousYear) then
                           Refresh
                        Else
                           DrawFocusFrame(m_CurrentDateIndex);
                     End;
          VK_Down : Begin
                        NextWeek;
                        if (m_CurrentMonth <> m_PreviousMonth) or
                           (m_CurrentYear <> m_PreviousYear) then
                           Refresh
                        Else
                           DrawFocusFrame(m_CurrentDateIndex);
                     End;
          VK_Prior: Begin
                        PrevMonth;
                        Refresh;
                     End;
          Vk_Next : Begin
                        NextMonth;
                        Refresh;
                     End;

          VK_Home : Begin
                        NextYear;
                        Refresh;
                     End;
          VK_End : Begin
                        PrevYear;
                        Refresh;
                     End;
          VK_Return: Begin
                        TIsamDateEdit( ctlParent ).Date := m_CurrentDateSelected;
                        ModalResult := 1;
                     End;
          VK_Escape : FormCancel;
     Else

     End;
End;


{********************** Form Mouse Down *******************
 **********************************************************}
Procedure TItabCalPop.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
Var
  nIndex : Integer;
  Key: Word;
Begin
  {Check if mouse was pressed in Left button area}
  if PointInRect(GetLeftButtonRect, X, Y) then
     Begin
        Key := Vk_Prior;
        FormKeyDown(Sender, Key,Shift);
     End;

  {Check if mouse was pressed in Right button area}
  if PointInRect(GetRightButtonRect, X, Y) then
     Begin
        Key := Vk_Next;
        FormKeyDown(Sender, Key,Shift);
     End;

  {Check if mouse was pressed in date area}
  if PointInRect(GetCalendarRect, X, Y) then
     Begin
        m_MouseDown := True;
        nIndex := GetIndexFromPoint( X, Y );

        if (nIndex >= GetMonthBegin) and
           (nIndex < (DaysInMonth(m_CurrentMonth) + GetMonthBegin)) then
           Begin
             SetDate(nIndex - m_CurrentDateIndex);
             DrawFocusFrame(nIndex);
           End
        Else
            m_MouseDown := False;

     End;
End;

{******************* Form Mouse Move **********************
 **********************************************************}
Procedure TItabCalPop.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
Var
  nIndex : Integer;
Begin
    if m_MouseDown = True then
       Begin
          if PointInRect(GetCalendarRect, X, Y) then
          Begin
               nIndex := GetIndexFromPoint( X, Y );
               if (nIndex >= GetMonthBegin) and
                  (nIndex < (DaysInMonth(m_CurrentMonth) + GetMonthBegin)) and
                  (nIndex <> m_CurrentDateIndex) then
               Begin
                    SetDate(nIndex - m_CurrentDateIndex);
                    DrawFocusFrame(nIndex);
               End;
          End;
       End;
End;


{******************* Form Mouse Up ************************
 **********************************************************}
Procedure TItabCalPop.FormMouseUp( Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer );
Var
   TempRect : Trect;
Begin
  if m_MouseDown = True then
     Begin
        m_MouseDown := False;
        TIsamDateEdit( ctlParent ).Date := m_CurrentDateSelected;
        TIsamDateEdit( ctlParent ).Text := DatetoStr(m_CurrentDateSelected);
        ModalResult := 1;
     End;
End;


{********************** Form Paint ************************
 **********************************************************}
Procedure TItabCalPop.FormPaint(Sender: TObject);
Begin
  DrawCalendarBorder;
  DrawMonthHeader;
  DrawDaysHeader;
  DrawDates;
  DrawButtons;
  DrawFocusFrame(m_CurrentDateIndex);
End;


{********************* Get Left Button Rectangle ******************
 ***** Get the rectangle used for the left button.            *****
 ******************************************************************}
Function TItabCalPop.GetLeftButtonRect: TRect;
Var
  TempRect: TRect;
Begin
   {Define Left Button Rectangle}
   TempRect.Top := ClientRect.Top + BORDER;
   TempRect.Bottom := TempRect.Top + BUTTON_WIDTH;
   TempRect.Left := ClientRect.Left + BORDER + TEXT_INDENT;
   TempRect.Right := TempRect.Left + BUTTON_WIDTH;

   Result := TempRect;
End;

{******************** Get Right Button Rectangle ******************
 ***** Get the rectangle used for the right button.           *****
 ******************************************************************}
Function TItabCalPop.GetRightButtonRect: TRect;
Var
  TempRect: TRect;
Begin
   {Define Right Button Rectangle}
   TempRect.Top := ClientRect.Top + BORDER;
   TempRect.Bottom := TempRect.Top + BUTTON_WIDTH;
   TempRect.Right := BORDER + TEXT_INDENT + (m_FontWidth * 3 * 7);
   TempRect.Left := TempRect.Right - BUTTON_WIDTH;

   Result := TempRect;
End;

{********************** Get Calendar Rectangle ********************
 ***** Get the rectangle used for the calendar section        *****
 ******************************************************************}
Function TItabCalPop.GetCalendarRect: TRect;
Var
  TempRect: TRect;
Begin
  TempRect := ClientRect;

  With TempRect do
     Begin
        Left := BORDER + TEXT_INDENT;
        Top := ( m_FontHeight * 2 ) + BORDER;
        Bottom := Top + ( m_FontHeight * 6 );
        Right := Left + ( 7 * ( m_fontWidth * 3 ) );
     End;

  Result := TempRect;
End;


{******************** Get Rectangle From Index ********************
 ***** Get the rectangle used for the calendar section        *****
 ******************************************************************}
Function TItabCalPop.GetRectFromIndex(nIndex : Integer): TRect;
Var
  TempRect: TRect;
  nWeek : Integer;
  nDay : Integer;
Begin
  TempRect := GetCalendarRect;

  With TempRect do
     Begin
        Case nIndex of
             1..7 :  nWeek := 1;
             8..14:  nWeek := 2;
             15..21: nWeek := 3;
             22..28: nWeek := 4;
             29..35: nWeek := 5;
             36..42: nWeek := 6;
        End;

        nDay := nIndex - ((nWeek-1) *7);

        Left := Left + ((m_FontWidth * 3) * (nDay-1));
        Top := (m_FontHeight * nWeek ) + m_FontHeight + BORDER;
        Bottom := Top +  m_FontHeight ;
        Right := Left + m_fontWidth * 3;
     End;

  Result := TempRect;
End;

{*************************** Get Month Begin **************************
 ***** This function Gets the index value of the first day of the *****
 ***** month.                                                     *****
 ********************************************************************** }
Function TItabCalPop.GetMonthBegin: Integer;
Var
  FirstDate: TDateTime;
Begin
  FirstDate := EncodeDate( m_CurrentYear, m_CurrentMonth, 1 );
  Result := DayOfWeek( FirstDate ); { day of week for 1st of month }
End;


{********************** Is Leap Year **********************
 **********************************************************}
Function TItabCalPop.IsLeapYear: Boolean;
Begin
  Result := ( m_CurrentYear mod 4 = 0 ) and
            ( ( m_CurrentYear mod 100 <> 0 ) or ( m_CurrentYear mod 400 = 0 ) );
End;


{********************** LoadDateArray *********************
 **********************************************************}
Procedure TItabCalPop.LoadDateArray;
Var
  nIndex : Integer;
  nBeginIndex, nEndIndex : Integer;
Begin
  nBeginIndex := GetMonthBegin;
  nEndIndex := nBeginIndex + DaysInMonth(m_CurrentMonth) - 1;
  for nIndex := 1 to 42 do
  Begin
     if ( nIndex < nBeginIndex ) or ( nIndex > nEndIndex ) then
        m_DateArray[nIndex] := '  '
     Else
        m_DateArray[nIndex] := IntToStr( ( nIndex - nBeginIndex ) + 1 );
  End;
End;


{******************** Get Index From Date *****************
 **********************************************************}
Function TItabCalPop.GetIndexFromDate : Integer;
Begin
     Result := m_CurrentDay + GetMonthBegin;
End;


{****************** Get Index From Point ******************
 **********************************************************}
Function TItabCalPop.GetIndexFromPoint(nLeft : Integer ; nTop : Integer) : Integer;
Var
  nIndex, nWeek, nDay: Integer;
  nResult: Real;
  TempRect: Trect;
Begin
  TempRect := GetCalendarRect;

  nIndex := -1;
  {Is point in the calendar rectangle?}
  if ( nLeft > TempRect.Left ) and ( nTop > TempRect.Top ) and
      ( nLeft < TempRect.Right ) and ( nTop < TempRect.Bottom ) then
     Begin

        { Determine the week number of the selected date }
        nResult := ( nTop - BORDER ) / ( m_FontHeight ) - 1;
        nWeek := Trunc( nResult );

        { Adjust Date Rect }
        TempRect.Top := TempRect.Top + ( ( nWeek - 1 ) * m_FontHeight );
        TempRect.Bottom := TempRect.Top + m_FontHeight;
        TempRect.Left := BORDER + TEXT_INDENT;
        TempRect.Right := TempRect.Left + m_FontWidth * 3;

        { Determine the day number of the selected date }
        for nDay := 1 to 7 do        {Cycle through the days}
           Begin
              nIndex := nDay + ( ( nWeek - 1 ) * 7 );
              if ( nLeft >= TempRect.Left ) and ( nLeft <= TempRect.Right ) then
                 break
              Else
                 Begin
                    TempRect.Left := TempRect.Right;
                    TempRect.Right := TempRect.Left + m_FontWidth * 3;
                 End;
           End;
     End;
  Result := nIndex;
End;


{******************** Get Previous Day ********************
 **********************************************************}
Procedure TItabCalPop.PrevDay;
Begin
    SetDate(-1);
End;


{********************* Get Next Day ***********************
 **********************************************************}
Procedure TItabCalPop.NextDay;
Begin
     SetDate(1);
End;


{******************** Get Previous Week *******************
 **********************************************************}
Procedure TItabCalPop.PrevWeek;
Begin
    SetDate(-7);
End;


{******************** Get Next Week ***********************
 **********************************************************}
Procedure TItabCalPop.NextWeek;
Begin
     SetDate(7);
End;


{******************** GetPreviousMonth ********************
 **********************************************************}
Procedure TItabCalPop.PrevMonth;
Var
   nDays : Integer;
   nMonth : Integer;
Begin
  if m_CurrentMonth > 1 then
     nMonth := m_CurrentMonth - 1
  Else
     nMonth := 12;
  nDays := DaysInMonth(nMonth);
  SetDate(-nDays);
End;


{******************** Get Next Month **********************
 **********************************************************}
Procedure TItabCalPop.NextMonth;
Begin
  SetDate(DaysInMonth(m_CurrentMonth));
End;

{GetNextYear}
Procedure TItabCalPop.NextYear;
Begin

  {If the current year is a leap year and the date is
   before February 29, add 1 day}
  if IsLeapYear and (m_CurrentMonth < 3) then
     SetDate(1);

  SetDate(365);
  {If the current year is a leap year and the date is
   after February 29, add 1 day}
  if IsLeapYear and (m_CurrentMonth > 3) then
     SetDate(1);
End;


{******************* GetPrevious Year *********************
 **********************************************************}
Procedure TItabCalPop.PrevYear;
Begin
  {If the current year is a leap year and the date is
   after February 29, subtract 1 day}
  if IsLeapYear and (m_CurrentMonth > 3) then
     SetDate(-1);

  SetDate(-365);
  {If the Previous year is a leap year and the date is
   before February 29, subtract 1 day}
  if IsLeapYear and (m_CurrentMonth < 3) then
     SetDate(-1);
End;


{***************** Set Date **************************
 **** This procedure adjusts the date by nDays    ****
 **** nDays can be possitive or negative.  It     ****
 **** also populates the vars YEAR, MONTH and DAY ****
 *****************************************************}
Procedure TItabCalPop.SetDate(nDays : Integer);
Begin
   {Save current date information}
   m_PreviousDateIndex := m_CurrentDateIndex;
   DecodeDate(m_CurrentDateSelected,m_PreviousYear,m_PreviousMonth,m_PreviousDay);

   {Change the date and update member variables}
   m_CurrentDateSelected := m_CurrentDateSelected + nDays;
   DecodeDate(m_CurrentDateSelected,m_CurrentYear,m_CurrentMonth,m_CurrentDay);
   m_CurrentDateIndex := ( m_CurrentDay + GetMonthBegin ) - 1;

   {Reload Date Array if month or year changed}
   if (m_CurrentMonth <> m_PreviousMonth) or (m_CurrentYear <> m_PreviousYear)then
      LoadDateArray;
End;

Procedure TItabCalPop.FormKeyPress(Sender: TObject; Var Key: Char);
Begin
  if (Key = 'H') or (Key = 'h') then Begin
    m_CurrentDateSelected := Date;
    DecodeDate( m_CurrentDateSelected, m_CurrentYear, m_CurrentMonth, m_CurrentDay );
    m_CurrentDateIndex := m_CurrentDay + GetMonthBegin - 1;
    m_PreviousDateIndex := 0;
    LoadDateArray;
    Refresh;
  End;
End;

End.
