Unit Itabedit;
Interface
{copyright 1995,1996,1997 by Stellberg Computer GmbH}
Uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Menus, ITable,
  Buttons;

Type
{$IFNDEF VER90}
  ShortString     = String;
{$ENDIF}
  StrCaseSet = (scNormal,scUpper,scLower,scFirstUp);
  HTextStyle = (tsNone, tsRaised, tsRecessed);
  TDrawStyle = (dsStandard, dsCtl3D, dsIsam);

{ TISAMEdit }

   TISAMEdit = Class(TCustomEdit)
   Private
      FMax : extended;
      FMin : extended;
      FCtrlLink: TITblCtrlLink;
      FDataSource: TIsamDatasource;
      FCanvas: TControlCanvas;
      FAlignment: TAlignment;
      FFocused: Boolean;
      FTextMargin: Integer;
      FStrCase: StrCaseSet;
      FLastChar: Char;
      FEnterExits: Boolean;
      FModified: Boolean;
      FEntered: Boolean;
      FNoDupKey : Boolean;
      FKeyNoDup : Byte;
      FRightNull : Boolean;
      FValue : extended;
      FValidate : Boolean;
      Procedure CalcTextMargin;
      Procedure DoTakeAction(Action: ISAMEvent; Info: Longint);
      Procedure EditingChange(Sender: TObject);
      Function  GetDataField: ShortString;
      Function  GetReadOnly: Boolean;
      Procedure SetDataField(Const Value: ShortString);
      Procedure SetFocused(Value: Boolean);
      Procedure SetReadOnly(Value: Boolean);
      Procedure SetStringCase(sc: StrCaseSet);
      Procedure SetDataSource(Value: TIsamDataSource);
      Procedure SetValue(Value : extended);
      Procedure SetValidate(Value : Boolean);
      Function GetDatasource: TIsamDataSource;
      Procedure UpdateData(Sender: TObject);
      Procedure WMCut(Var Message: TMessage); Message WM_CUT;
      Procedure WMPaste(Var Message: TMessage); Message WM_PASTE;
      Procedure CMEnter(Var Message: TCMEnter); Message CM_ENTER;
      Procedure WMPaint(Var Message: TWMPaint); Message WM_PAINT;
      Procedure CMFontChanged(Var Message: TMessage); Message CM_FONTCHANGED;
   Protected
      Procedure DataChange(Sender: TObject); Virtual;
      Procedure FormatText; Dynamic;
      Procedure UnFormatText; Dynamic;
      Procedure Change; Override;
      Procedure Click; Override;
      Procedure CMExit(Var Message: TCMExit); Message CM_Exit;
      Procedure KeyDown(Var Key: Word; Shift: TShiftState); Override;
      Procedure KeyPress(Var Key: Char); Override;
      Procedure Notification(AComponent: TComponent;
                             Operation: TOperation); Override;
      Property Modified: Boolean Read FModified Write FModified;
      Property Validate : Boolean Read FValidate Write SetValidate;
      Property Value : extended Read FValue Write SetValue;
   Public
      Constructor Create(AOwner: TComponent); Override;
      Destructor Destroy; Override;
      Function Valid ( Value : extended ) : Boolean; Dynamic;
   Published
      Property RightNull: Boolean Read FRightNull Write FRightNull Default False;
      Property IsamNoDupKey : Boolean Read FNoDupKey Write FNoDupKey;
      Property IsamKeyNoDup : Byte Read FKeyNoDup Write FKeyNoDup Default 1;
      Property Alignment: TAlignment Read FAlignment Write FAlignment;
      Property AutoSelect;
      Property AutoSize;
      Property BorderStyle;
      Property Color;
      Property Ctl3D;
      Property DataSource: TIsamDataSource Read GetDataSource Write SetDataSource;
      Property ISAMField: ShortString Read GetDataField Write SetDataField;
      Property DragCursor;
      Property DragMode;
      Property Enabled;
      Property EnterExits: Boolean Read FEnterExits Write FEnterExits Default True;
      Property Font;
      Property MaxLength;
      Property OEMConvert;
      Property ParentColor;
      Property ParentCtl3D;
      Property ParentFont;
      Property ParentShowHint;
      Property PasswordChar;
      Property PopupMenu;
      Property ReadOnly: Boolean Read GetReadOnly Write SetReadOnly
                                                  Default False;
      Property ShowHint;
      Property StringCase: StrCaseSet Read FStrCase Write SetStringCase;
      Property TabOrder;
      Property TabStop;
      Property Visible;
      Property OnChange;
      Property OnClick;
      Property OnDblClick;
      Property OnDragDrop;
      Property OnDragOver;
      Property OnEndDrag;
      Property OnEnter;
      Property OnExit;
      Property OnKeyDown;
      Property OnKeyPress;
      Property OnKeyUp;
      Property OnMouseDown;
      Property OnMouseMove;
      Property OnMouseUp;
   End;

  TIsamCustomStrEdit = Class (TIsamEdit)
  Private
    FAlignment: TAlignment;
    FOldAlignment : TAlignment;
    FTextMargin : Integer;
    FRightNull  : Boolean;
    Function CalcTextMargin : Integer;
    Procedure CMEnter(Var Message: TCMEnter); Message CM_ENTER;
    Procedure CMExit(Var Message: TCMExit);   Message CM_Exit;
    Procedure WMPaint(Var Message: TWMPaint); Message WM_PAINT;
    Procedure SetAlignment(Value: TAlignment);
  Protected
    Property Alignment: TAlignment Read FAlignment Write SetAlignment Default taRightJustify;
    Property RightNull: Boolean Read FRightNull Write FRightNull Default False;
    Procedure FormatText; Dynamic;
    Procedure UnFormatText; Dynamic;
  Public
    Constructor Create(AOwner: TComponent); Override;
  End;

  TIsamStrEdit = Class (TIsamCustomStrEdit)
  Published
    Property Alignment;
    Property AutoSize;
    Property BorderStyle;
    Property CharCase; 
    Property CoLor;
    Property Ctl3D;
    Property DragCursor;
    Property DragMode;
    Property Enabled;
    Property Font;
    Property HideSelection;
    Property MaxLength;
    Property ParentColor;
    Property ParentCtl3D;
    Property ParentFont;
    Property ParentShowHint;
    Property PopupMenu;
    Property ReadOnly;
    Property RightNull;
    Property ShowHint;
    Property TabOrder;
    Property Visible;
    Property OnChange;
    Property OnClick;
    Property OnDblClick;
    Property OnDragDrop;
    Property OnDragOver;
    Property OnEndDrag;
    Property OnEnter;
    Property OnExit;
    Property OnKeyDown;
    Property OnKeyPress;
    Property OnKeyUp;
    Property OnMouseDown;
    Property OnMouseMove;
    Property OnMouseUp;
  End;

Type
  TNumericType = (ntGeneral, ntCurrency, ntPercentage);
  TMaskString = String [25];

{ mask component }
Type
  TMasks = Class (TPersistent)
  Private
    FPositiveMask : TMaskString;
    FNegativeMask : TMaskString;
    FZeroMask : TMaskString;
    FOnChange: TNotifyEvent;
  Protected
    Procedure SetPositiveMask (Value : TMaskString);
    Procedure SetNegativeMask (Value : TMaskString);
    Procedure SetZeroMask (Value : TMaskString);
  Public
    Constructor Create;
    Property OnChange: TNotifyEvent Read FOnChange Write FOnChange;
  Published
    Property PositiveMask : TMaskString Read FPositiveMask Write SetPositiveMask;
    Property NegativeMask : TMaskString Read FNegativeMask Write SetNegativeMask;
    Property ZeroMask : TMaskString Read FZeroMask Write SetZeroMask;
  End;

{ num edit component }
Type
  TIsamCustomNumEdit = Class (TIsamCustomStrEdit)
  Private
    FDecimals : Word;
    FDigits : Word;
    FMasks : TMasks;
    FMax : extended;
    FMin : extended;
    FNumericType : TNumericType;
    FUseRounding : Boolean;
    FValue : extended;
    FValidate : Boolean;
    Procedure CMEnter(Var Message: TCMEnter); Message CM_ENTER;
    Procedure CMExit(Var Message: TCMExit);   Message CM_Exit;
    Procedure SetDecimals(Value : Word);
    Procedure SetDigits(Value : Word);
    Procedure SetMasks (Mask : TMasks);
    Procedure SetMax(Value : extended);
    Procedure SetMin(Value : extended);
    Procedure SetNumericType(Value : TNumericType);
    Procedure SetValue(Value : extended);
    Procedure SetValidate(Value : Boolean);
  Protected
    Procedure DataChange(Sender: TObject); Override;
    Procedure FormatText; Dynamic;
    Procedure KeyPress(Var Key: Char); Override;
    Procedure UnFormatText; Dynamic;
    Property Decimals : Word Read FDecimals Write SetDecimals;
    Property Digits : Word Read FDigits Write SetDigits;
    Property Masks : TMasks Read FMasks Write SetMasks;
    Property Max : extended Read FMax Write SetMax;
    Property Min : extended Read FMin Write SetMin;
    Property NumericType : TNumericType Read FNumericType Write SetNumericType Default ntCurrency;
    Property UseRounding : Boolean Read FUseRounding Write FUseRounding;
    Property Value : extended Read FValue Write SetValue;
    Property Validate : Boolean Read FValidate Write SetValidate;
  Public
    Constructor Create(AOwner: TComponent); Override;
    Destructor Destroy; Override;
    Function AsDouble : double; Dynamic;
    Function AsInteger : Integer; Dynamic;
    Function AsLongint : Longint; Dynamic;
    Function AsReal : real; Dynamic;
    Function AsString : ShortString; Dynamic;
    Procedure MaskChanged ( Sender : TObject );
    Function Valid ( Value : extended ) : Boolean; Dynamic;
  End;

  TIsamDateButton = Class( TBitBtn )
  Protected
    Procedure Click; Override;
  End;


  TISAMDateEdit = Class(TIsamEdit)
   Private
     hBitmap: HBitmap;
     FButton: TIsamDateButton;
     FDate: TDateTime;
  Protected
     Function GetDate: TDateTime;
     Procedure SetDate( dtArg: TDateTime );
     Procedure DataChange(Sender: TObject); Override;
     Procedure WMSize( Var Message: TWMSize ); Message WM_SIZE;
     Procedure KeyPress( Var Key: Char ); Override;
  Public
     Constructor Create( AOwner: TComponent ); Override;
     Destructor Destroy; Override;
     Procedure CreateParams( Var Params: TCreateParams ); Override;
     Property Date: TDateTime Read GetDate Write SetDate;
  End;


  TIsamNumEdit = Class (TIsamCustomNumEdit)
  Published
    Property AutoSize;
    Property BorderStyle;
    Property Color;
    Property Ctl3D;
    Property Decimals;
    Property Digits;
    Property DragCursor;
    Property DragMode;
    Property Enabled;
    Property Font;
    Property HideSelection;
    Property Masks;
    Property Max;
    Property Min;
    Property NumericType;
    Property ParentColor;
    Property ParentCtl3D;
    Property ParentFont;
    Property ParentShowHint;
    Property PopupMenu;
    Property ReadOnly;
    Property ShowHint;
    Property TabOrder;
    Property UseRounding;
    Property Value;
    Property Validate;
    Property Visible;
    Property OnChange;
    Property OnClick;
    Property OnDblClick;
    Property OnDragDrop;
    Property OnDragOver;
    Property OnEndDrag;
    Property OnEnter;
    Property OnExit;
    Property OnKeyDown;
    Property OnKeyPress;
    Property OnKeyUp;
    Property OnMouseDown;
    Property OnMouseMove;
    Property OnMouseUp;
  End;

Var  FNew         : Boolean;

Implementation
Uses Filer, ITabCalP, IniFiles, ITblCons, ITabTool;
{$IFDEF VER90}
{$R ITABDATE.R32}
{$ELSE}
{$R ITABDATE.R16}
{$ENDIF}

Type
  TSetOfChar = Set of Char;
Var
  OldMaxLength : Integer;


{========================================================================}
{ support routines                                                       }
{========================================================================}

Function Power ( X, Y : Integer ) : real;
Begin
  Result := exp ( ln ( X ) * Y );
End;

Function FirstUpChar(chprev, ch: Char): Char;
Var
  c : Char;
Begin
   C := ' ';
   if chprev in [#0,#$20..#$26,#$28..#$2F,#$3A..#$40,#$5B..#$5F,#$7B..#$7F] then
   Begin
      Case ch of
      '' : C := '';
      '' : C := '';
      '' : C := '';
        Else C := UpCase(ch)
     End; {of Case}
   End Else c := ch;
   Result := c;
End;

Function StripChars ( Const Text : ShortString; ValidChars : TSetOfChar ) : ShortString;
Var
  S : ShortString;
  i : Integer;
  Negative : Boolean;
Begin
  Negative := False;
  if (Text [ 1 ] = '-') or (Text [length (Text)] = '-' ) then
    Negative := True;
  S := '';
  for i := 1 to length ( Text ) do
    if Text [ i ] in ValidChars then
      S := S + Text [ i ];
  if Negative then
    Result := '-' + S
  Else
    Result := S;
End;

{ TISAMEdit }

Constructor TISAMEdit.Create(AOwner: TComponent);
Begin
   Inherited Create(AOwner);
   FCtrlLink := TITblCtrlLink.Create(Self);
   FCtrlLink.FOnAnyEvent := DoTakeAction;
   FDataSource:= Nil;
   CalcTextMargin;
   FKeyNoDup := 1;
   FEnterExits := True;
   FNew := False;
   Text:= '';
End;

Destructor TISAMEdit.Destroy;
Begin
   SetDatasource(Nil);
   FCtrlLink.Free;
   FCtrlLink := Nil;
   FCanvas.Free;
   FNew := False;
   Inherited Destroy;
End;

Function TIsamEdit.Valid ( Value : extended ) : Boolean;
Const F:ShortString = 'Der eingegebene Wert mu zwischen %g und %g liegen';
Var S,FStr,MStr : ShortString;
Begin
  Result := True;
  if Validate and (( Value < FMin ) or ( Value > FMax )) then
    Begin
    {$IFDEF VER90}
    Str(FMin,FStr);
    Str(FMax,MStr);
    S:= 'Der eingegebene Wert mu zwischen '+FStr+' und '+MStr+' liegen.';
    {$ELSE}
    FmtStr(S,F,[FMin,FMax]);
    {$ENDIF}
    MessageDlg(S,mtError, [mbOk], 0);
    Result := False;
    End;
End;

Procedure TIsamEdit.SetValidate(Value : Boolean);
Begin
  if FValidate <> Value then
    Begin
    FValidate:= Value;
    if FValidate and (( FValue < FMin ) or ( FValue > FMax )) then
      Begin
      FValue := FMin;
      FormatText;
      End;
    End;
End;

Procedure TIsamEdit.SetValue(Value : extended);
Begin
  if ( FValue <> Value ) and ( Valid ( Value ) ) then
    Begin
    FValue := Value;
    FormatText;
    End;
End;

Procedure TIsamEdit.UnFormatText;
Begin
  Text := StripChars ( Text, [ '0'..'9', DecimalSeparator, ThousandSeparator ] );
  if Value < 0 then
    Text := '-' + Text;
End;

Procedure TIsamEdit.FormatText;
Var Txt: ShortString;
Begin
  Txt:= Text;
  While Length(Txt) < MaxLength do Txt:= '0'+Txt;
  Text:= Txt;
End;

Procedure TISAMEdit.Notification(AComponent: TComponent;
   Operation: TOperation);
Begin
   Inherited Notification(AComponent, Operation);
   if (Operation = opRemove) and (AComponent = FDatasource) then
      SetDatasource(Nil);
End;

Procedure TISAMEdit.DoTakeAction(Action: ISAMEvent; Info: Longint);
Begin
   Case Action of
      deDataChange    : Begin
                          if FNew = False then DataChange(Self);
                        End;
      deDataSetScroll : Begin
                          DataChange(Self);
                        End;
      deFieldChange   : Begin
                          if (info = 0) or
                          (Info = FCtrlLink.FieldNumber) then DataChange(Self);
                        End;
      deRecordChange  : Begin
                          DataChange(Self);
                        End;
      deTableChange   : Begin
                          if Datasource <> NIL then begin
                            if DataSource.DataSet <> NIL then begin
                              DataSource.DataSet.Get;
                              ReadOnly:= ReadOnly;
                              DataChange(Self);
                            end;
                          end;
                        End;
      deDataSetChange : Begin
                          if Datasource <> NIL then begin
                            if DataSource.DataSet <> NIL then begin
                              {DataSource.DataSet.Get;}
                              ReadOnly := ReadOnly;
                              DataChange(Self);
                            end;
                          end;
                        End;
      deUpdateState   : Begin
                          EditingChange(Self);
                          UpdateData(Self);
                        End;
      deUpdateRecord  : Begin
                          UpdateData(Self);
                        End;
      deClearFields   : Begin
                          Text:= '';
                          FNew:= True;
                        end;
   End;
End;

Procedure TISAMEdit.SetDatasource(Value: TIsamDatasource);
Begin
  if FDatasource <> Value then Begin
    FCtrlLink.Datasource := Value;
    FDatasource := FCtrlLink.Datasource;
  End;
End;

Function TIsamEdit.GetDatasource: TIsamDatasource;
Begin
  Result:= FCtrlLink.Datasource;
End;

Procedure TISAMEdit.SetStringCase(sc: StrCaseSet);
Begin
   FStrCase := sc;
   Case sc of
      scUpper  : CharCase := ecUpperCase;
      scLower  : CharCase := ecLowerCase;
   End;
End;

Procedure TISAMEdit.Click;
Begin
   Inherited Click;
   if FEntered and AutoSelect then SelectAll;
   FEntered := False;
End;

Procedure TISAMEdit.KeyDown(Var Key: Word; Shift: TShiftState);
Var
   v: Integer;
   vl: Integer;
Begin
   Inherited KeyDown(Key, Shift);
   if FCtrlLink.TableActive then
      if (Key = VK_DELETE) or ((Key = VK_INSERT) and (ssShift in Shift)) then
      Begin
         v := SelStart;
         vl := SelLength;
         FCtrlLink.Edit;
         SelStart := v;
         SelLength := vl;
      End;
End;

Procedure TISAMEdit.KeyPress(Var Key: Char);
Var
   i: Integer;
   v: Integer;
   vl: Integer;
Begin
   if FEnterExits then
   Begin
      if Key = #13 then
      Begin
         key := #0;
         PostMessage(TWinControl(Owner).Handle, WM_NEXTDLGCTL, 0, 0);
      End;
   End;
   if Key <> #0 then
   Begin
      if (FStrCase = scFirstUp) then
      Begin
         i := SelStart;
         if i = 0 then
            FLastChar := #0
         Else
            FLastChar := Text[i];
         Key := FirstUpChar(FLastChar, Key);
         FLastChar := Key;
      End;
   End;
   Inherited KeyPress(Key);
   if FCtrlLink.TableActive then
   Begin
      Case Key of
        ^H, ^V, ^X, #32..#255: Begin
                                  v := SelStart;
                                  vl := SelLength;
                                  FCtrlLink.Edit;
                                  SelStart := v;
                                  SelLength := vl;
                               End;
        #27: Begin
                Text := DataSource.DataSet.StringGetN(FCtrlLink.FieldNumber);
                SelectAll;
                Key := #0;
             End;
      End;
   End;
End;

Procedure TISAMEdit.SetFocused(Value: Boolean);
Begin
  if not assigned(FDatasource) then Exit;
  if FFocused <> Value then Begin
    FFocused := Value;
    if (FAlignment <> taLeftJustify) then Invalidate;
  End;
  if AutoSelect then SelectAll;
End;

Procedure TISAMEdit.Change;
Begin
   if FCtrlLink.TableActive then Begin
     if FCtrlLink.Edit then Begin
       Datasource.DataSet.Modified := True;
       Modified := True;
     End;
   End;
   Inherited Change;
End;

Function TISAMEdit.GetDataField: ShortString;
Begin
  Result := FCtrlLink.FieldName;
End;

Procedure TISAMEdit.SetDataField(Const Value: ShortString);
Begin
  FCtrlLink.FieldName := Value;
End;

Function TISAMEdit.GetReadOnly: Boolean;
Begin
   Result := Inherited ReadOnly;
   if (not Result) and (FCtrlLink.TableActive) then
   Begin
      Result := Datasource.DataSet.ReadOnly;
      Inherited ReadOnly := Result;
   End;
End;

Procedure TISAMEdit.SetReadOnly(Value: Boolean);
Begin
   Inherited ReadOnly := Value;
   if (not Value) and (FCtrlLink.TableActive) then
   Begin
      Inherited ReadOnly := Datasource.DataSet.ReadOnly;
   End;
End;

Procedure TISAMEdit.DataChange(Sender: TObject);
Var I   : Byte;
    L   : Longint;
    Ref : Longint;
    B   : Boolean;
Begin
  FCtrlLink.NoEdit := True;
  if FCtrlLink.TableActive then
  Begin
    FLastChar := #0;
    if DataSource.DataSet.FieldType(FCtrlLink.FieldNumber) = ftDate then
      MaxLength := 10
    Else if DataSource.DataSet.FieldType(FCtrlLink.FieldNumber) = ftWordDate then
      MaxLength := 10
    Else if DataSource.DataSet.FieldType(FCtrlLink.FieldNumber) <> ftDate then
      MaxLength := DataSource.DataSet.FieldLen(FCtrlLink.FieldNumber);
    Text := DataSource.DataSet.StringGetN(FCtrlLink.FieldNumber);
    if FNoDupkey then if FNew then Begin
      if Text = '' then Begin
        B := False;
        I := DataSource.Dataset.FieldLen(FCtrlLink.FieldNumber);
        L := BTUsedKeys(DataSource.Dataset.IfbPtr,FKeyNoDup);
        Repeat
          if Rightnull then
          Text := ITabTool.RightNull(IntStr(L),i) Else
          Text := ITabTool.F(IntStr(L),i);
          if BtKeyExists(DataSource.Dataset.IfbPtr,FKeyNoDup,Ref,Text) then inc(L) Else B := True;
        Until B;
        Datasource.DataSet.StringPutN(FCtrlLink.FieldNumber,Text);
        PostMessage(TWinControl(Owner).Handle, WM_NEXTDLGCTL, 0, 0);
        Modified := True;
        FCtrlLink.NoEdit := True;
      End;
    End;
  End
  Else Begin
    MaxLength := 0;
    if csDesigning in ComponentState then
    Text := Name Else
    Text := '';
  End;
  Modified := False;
  FCtrlLink.NoEdit := False;
End;

Procedure TISAMEdit.EditingChange(Sender: TObject);
Begin
   Inherited ReadOnly := not (FCtrlLink.InEdit);
   Modified := False;
End;

Procedure TISAMEdit.UpdateData(Sender: TObject);
Begin
   if FCtrlLink.TableActive and Modified then
   Begin
      Modified := False;
      Datasource.DataSet.StringPutN(FCtrlLink.FieldNumber,Text);
   End;
End;

Procedure TISAMEdit.WMPaste(Var Message: TMessage);
Begin
   FCtrlLink.Edit;
   Inherited;
End;

Procedure TISAMEdit.WMCut(Var Message: TMessage);
Begin
   FCtrlLink.Edit;
   Inherited;
End;

Procedure TISAMEdit.CMEnter(Var Message: TCMEnter);
Begin
   if FRightNull then UnformatText;
   SetFocused(True);
   Inherited;
   FEntered := True;
End;

Procedure TISAMEdit.CMExit(Var Message: TCMExit);
Begin
   if FRightNull then FormatText;
   DoExit;
   if not Focused then
   Begin
      if FCtrlLink.TableActive then UpDateData(Self);
      SetFocused(False);
   End;
End;

Procedure TISAMEdit.WMPaint(Var Message: TWMPaint);
Var
   Width, Indent, Left: Integer;
   R: TRect;
   DC: HDC;
   PS: TPaintStruct;
   S: ShortString;
Begin
   if (FAlignment = taLeftJustify) or FFocused then
   Begin
      Inherited;
      Exit;
   End;
{ Since edit controls do not handle justification unless multi-line (and
  then only poorly) we will draw right and center justify manually unless
  the edit has the focus. }
   if FCanvas = Nil then
   Begin
      FCanvas := TControlCanvas.Create;
      FCanvas.Control := Self;
   End;
   DC := Message.DC;
   if DC = 0 then DC := BeginPaint(Handle, PS);
   FCanvas.Handle := DC;
   Try
      FCanvas.Font := Font;
      With FCanvas do
      Begin
         R := ClientRect;
         if (BorderStyle = bsSingle) then
         Begin
            Brush.Color := clWindowFrame;
            FrameRect(R);
            InflateRect(R, -1, -1);
         End;
         Brush.Color := Color;
         S := Text;
         Width := TextWidth(S);
         if BorderStyle = bsNone then Indent := 0 Else Indent := FTextMargin;
         if FAlignment = taRightJustify then
           Left := R.Right - Width - Indent Else
           Left := (R.Left + R.Right - Width) div 2;
         TextRect(R, Left, Indent, S);
      End;
      Finally
      FCanvas.Handle := 0;
      if Message.DC = 0 then EndPaint(Handle, PS);
   End;
End;

Procedure TISAMEdit.CMFontChanged(Var Message: TMessage);
Begin
   Inherited;
   CalcTextMargin;
End;

Procedure TISAMEdit.CalcTextMargin;
Var
   DC: HDC;
   SaveFont: HFont;
   I: Integer;
   SysMetrics, Metrics: TTextMetric;
Begin
   DC := GetDC(0);
   GetTextMetrics(DC, SysMetrics);
   SaveFont := SelectObject(DC, Font.Handle);
   GetTextMetrics(DC, Metrics);
   SelectObject(DC, SaveFont);
   ReleaseDC(0, DC);
   I := SysMetrics.tmHeight;
   if I > Metrics.tmHeight then I := Metrics.tmHeight;
   FTextMargin := I div 4;
End;

{========================================================================}
{ Custom String Edit                                                     }
{========================================================================}

Constructor TIsamCustomStrEdit.Create(AOwner: TComponent);
Begin
  Inherited Create(AOwner);
  FAlignment := taLeftJustify;
  FTextMargin := CalcTextMargin;
End;

Function TIsamCustomStrEdit.CalcTextMargin : Integer;
Var
  DC: HDC;
  SaveFont: HFont;
  I: Integer;
  SysMetrics, Metrics: TTextMetric;
Begin
  DC := GetDC(0);
  GetTextMetrics(DC, SysMetrics);
  SaveFont := SelectObject(DC, Font.Handle);
  GetTextMetrics(DC, Metrics);
  SelectObject(DC, SaveFont);
  ReleaseDC(0, DC);
  I := SysMetrics.tmHeight;
  if I > Metrics.tmHeight then
    I := Metrics.tmHeight;
  Result := I div 4;
End;

Procedure TIsamCustomStrEdit.SetAlignment(Value: TAlignment);
Begin
  if FAlignment <> Value then
    Begin
    FAlignment := Value;
    Invalidate;
    End;
End;

Procedure TIsamCustomStrEdit.CMEnter(Var Message: TCMEnter);
Begin
  if FRightNull then UnformatText;
  Inherited;
  FOldAlignment := FAlignment;
  Alignment := taLeftJustify;
End;

Procedure TIsamCustomStrEdit.CMExit(Var Message: TCMExit);
Begin
  if FRightNull then FormatText;
  Inherited;
  Alignment := FOldAlignment;
End;

Procedure TIsamCustomStrEdit.UnformatText;
Begin
  Text := StripChars ( Text, [ '0'..'9', DecimalSeparator, ThousandSeparator ] );
End;

Procedure TIsamCustomStrEdit.FormatText;
Var Txt: ShortString;
Begin
  Txt:= Text;
  While Length(Txt) < MaxLength do Txt:= '0'+Txt;
  Text:= Txt;
End;

Procedure TIsamCustomStrEdit.WMPaint(Var Message: TWMPaint);
{paints the text in the appropriate position}
Var
  Width, Indent, Left: Integer;
  R: TRect;
  DC: HDC;
  PS: TPaintStruct;
  S: ShortString;
  Canvas: TControlCanvas;
Begin
  {let the existing code handle left justify}
  if (FAlignment = taLeftJustify) then
    Begin
    Inherited;
    Exit;
    End;

  Try
    Canvas := TControlCanvas.Create;
    Canvas.Control := Self;
    DC := Message.DC;
    if DC = 0 then
      DC := BeginPaint(Handle, PS);
    Canvas.Handle := DC;

    Canvas.Font := Font;
    With Canvas do
      Begin
      R := ClientRect;
      if (BorderStyle = bsSingle) then
        Begin
        Brush.Color := clWindowFrame;
        FrameRect(R);
        {$IFNDEF VER90}
        InflateRect(R, -1, -1);
        {$ENDIF}
        End;
      Brush.Color := Color;
      S := Text;
      Width := TextWidth(S);
      if BorderStyle = bsNone then
        Indent := 0
      Else
        Indent := FTextMargin;
      if FAlignment = taRightJustify then
        Left := R.Right - Width - Indent
      Else
        Left := (R.Left + R.Right - Width) div 2;
      TextRect(R, Left, Indent, S);
      End;
  Finally
    Canvas.Handle := 0;
    if Message.DC = 0 then
      EndPaint(Handle, PS);
  End;{try}
End;
{========================================================================}
{ Masks object                                                           }
{========================================================================}

Constructor TMasks.Create;
Begin
  Inherited Create;
  FPositiveMask := '#.##0';
  FNegativeMask := '';
  FZeroMask := '';
End;

Procedure TMasks.SetPositiveMask (Value : TMaskString);
Begin
  if FPositiveMask <> Value then
    Begin
    FPositIveMask := Value;
    OnChange(Self);
    End;
End;

Procedure TMasks.SetNegativeMask (Value : TMaskString);
Begin
  if FNegativeMask <> Value then
    Begin
    FNegativeMask := Value;
    OnChange(Self);
    End;
End;

Procedure TMasks.SetZeroMask (Value : TMaskString);
Begin
  if FZeroMask <> Value then
    Begin
    FZeroMask := Value;
    OnChange(Self);
    End;
End;

{========================================================================}
{ Custom Numeric Edit                                                    }
{========================================================================}

Constructor TIsamCustomNumEdit.Create(AOwner: TComponent);
Begin
  Inherited Create(AOwner);
  Width := 85;
  FAlignment := taRightJustify;
  FNumericType := ntCurrency;
  FDigits := 12;
  FDecimals := 2;
  AutoSelect := True;
  FMax := 0.0;
  FMin := 0.0;
  FValidate := False;
  FValue := 0.0;
  FormatText;
  FTextMargin := CalcTextMargin;
  FUseRounding := True;
  FMasks := TMasks.Create;
  FMasks.OnChange := MaskChanged;
  DecimalSeparator := '.';
  ThousandSeparator := ',';
End;

Destructor TIsamCustomNumEdit.Destroy;
Begin
  FMasks.Free;
  Inherited Destroy;
End;

Procedure TIsamCustomNumEdit.DataChange(Sender: TObject);
Begin
  FCtrlLink.NoEdit := True;
  if FCtrlLink.TableActive then Begin
    FLastChar := #0;
    if DataSource.DataSet.FieldType(FCtrlLink.FieldNumber) = ftInteger then Begin
      MaxLength := 10;
      Value:= DataSource.DataSet.IntegerGetN(FCtrlLink.FieldNumber);
    End
    Else if DataSource.DataSet.FieldType(FCtrlLink.FieldNumber) = ftReal then Begin
      MaxLength:= 12;
      Value:= DataSource.DataSet.RealGetN(FCtrlLink.FieldNumber);
    End
    Else Inherited DataChange(Sender);
  End
  Else Begin
    MaxLength := 0;
    if csDesigning in ComponentState then
    Text := Name Else
    Text := '';
  End;
  Modified := False;
  FCtrlLink.NoEdit := False;
End;

Function TIsamCustomNumEdit.AsInteger : Integer;
Const
  MaxInteger : Integer = 32767;
  MinInteger : Integer = -32768;
Begin
  Result := 0;
  if (FValue < MaxInteger) and  (FValue > MinInteger) then
    if FUseRounding then
      Result := round ( FValue )
    Else
      Result := trunc ( FValue );
End;

Function TIsamCustomNumEdit.AsLongint : Longint;
Const
  MaxLongint : Longint = 2147483647;
  MinLongint : Longint = -2147483647;
Begin
  Result := 0;
  if (FValue < MaxLongint ) and  (FValue > MinLongint) then
    if FUseRounding then
      Result := round ( FValue )
    Else
      Result := trunc ( FValue );
End;

Function TIsamCustomNumEdit.AsReal : real;
Const
  MaxReal : real = 1.7E38;
  MinReal : real = -1.7E38;
Begin
  Result := 0;
  if (FValue < MaxReal) and  (FValue > MinReal) then
     Result := FValue;
End;

Function TIsamCustomNumEdit.AsDouble : double;
Const
  MaxDouble : double = 1.7E308;
  MinDouble : double = -1.7E308;
Begin
  Result := 0;
  if (FValue < MaxDouble) and  (FValue > MinDouble) then
     Result := round ( FValue );
End;

Function TIsamCustomNumEdit.AsString : ShortString;
Const
  ValidChars = [ '0'..'9', ',', '.' ];
Begin
  Result := StripChars ( Text, ValidChars );
  if Value < 0 then
    Result := '-' + Result;
End;

Procedure TIsamCustomNumEdit.SetMasks (Mask : TMasks);
Begin
  if fMasks <> Mask then
    Begin
    fMasks := Masks;
    Invalidate;
    End;
End;

Procedure TIsamCustomNumEdit.SetMin(Value : extended);
Begin
  if FMin <> Value then
    Begin
    FMin := Value;
    if FValue < FMin then
      FValue := FMin;
    End;
End;

Procedure TIsamCustomNumEdit.SetMax(Value : extended);
Begin
  if FMax <> Value then
    Begin
    FMax := Value;
    if FValue > FMax then
      FValue := FMax;
    End;
End;

Procedure TIsamCustomNumEdit.SetValue(Value : extended);
Begin
  if ( FValue <> Value ) and ( Valid ( Value ) ) then
    Begin
    FValue := Value;
    FormatText;
    End;
End;

Procedure TIsamCustomNumEdit.SetDigits(Value : Word);
Begin
  if FDigits <> Value then
    Begin
    FDigits := Value;
    FormatText;
    End;
End;

Procedure TIsamCustomNumEdit.SetDecimals (Value : Word);
Var NStr: TMaskString;
    i   : Integer;
Begin
  if FDecimals <> Value then Begin
    FDecimals := Value;
    FormatText;
    if csDesigning in ComponentState then Begin
      NStr:= '';
      i:= 0;
      if FDecimals > Digits then Digits:= Decimals + 1;
      While i < Digits - Decimals - 1 do Begin
        NStr:= NStr + '#';
        Inc(i);
      End;
      NStr:= NStr + '0';
      if (Decimals > 0) then Begin
        NStr:= NStr + '.';
        i:= 0;
        While i < Decimals - 1 do Begin
          NStr:= NStr + '#';
          inc(i);
        End;
        NStr:= NStr + '0';
      End;
      Masks.PositiveMask:= NStr;
    End;
  End;
End;

Procedure TIsamCustomNumEdit.SetNumericType(Value: TNumericType);
Begin
  if FNumericType <> Value then
    Begin
    FNumericType := Value;
    FormatText;
    End;
End;

Procedure TIsamCustomNumEdit.SetValidate(Value : Boolean);
Begin
  if FValidate <> Value then
    Begin
    FValidate:= Value;
    if FValidate and (( FValue < FMin ) or ( FValue > FMax )) then
      Begin
      FValue := FMin;
      FormatText;
      End;
    End;
End;

Function TIsamCustomNumEdit.Valid ( Value : extended ) : Boolean;
Const F:ShortString = 'Der eingegebene Wert mu zwischen %g und %g liegen';
Var S,FStr,MStr : ShortString;
Begin
  Result := True;
  if Validate and (( Value < FMin ) or ( Value > FMax )) then
    Begin
    {$IFDEF VER90}
    Str(FMin,FStr);
    Str(FMax,MStr);
    S:= 'Der eingegebene Wert mu zwischen '+FStr+' und '+MStr+' liegen.';
    {$ELSE}
    FmtStr(S,F,[FMin,FMax]);
    {$ENDIF}
    MessageDlg(S,mtError, [mbOk], 0);
    Result := False;
    End;
End;

Procedure TIsamCustomNumEdit.KeyPress(Var Key: Char);
Begin
  {only allow numerics, commas and one period}
  if Key = ',' then Key:= '.';
  if (Key = DecimalSeparator) and (pos (DecimalSeparator, Text) = 0)
  and (Decimals > 0) then
    Begin
    Inherited KeyPress(Key);
    MaxLength := MaxLength + 1;
    End
  Else
  if ( Key = '-' ) and ( pos ( '-', Text ) = 0 ) then
    Begin
    Inherited KeyPress(Key);
    MaxLength := MaxLength + 1;
    End
  Else
  if Key in [ '0'..'9', ThousandSeparator, #8,#13 ] then
    Inherited KeyPress(Key)
  Else
    Key := #0;
End;

Procedure TIsamCustomNumEdit.CMEnter(Var Message: TCMEnter);
Begin
  {strip the mask and left justify the field}
  UnFormatText;
  OldMaxLength := MaxLength;
  MaxLength := FDigits;
  Inherited;
End;

Procedure TIsamCustomNumEdit.CMExit(Var Message: TCMExit);
Var
  S : String [80];
  X : extended;
Begin
  {format the Shortstring with the mask when leaving the field}
  MaxLength := OldMaxLength;
  S := StripChars (Text, [ '0'..'9', DecimalSeparator ]); {remove all literal characters}
  if S = '' then
    X := 0.0
  Else
    X := StrDez( S );
  if Valid ( X ) then
    Begin
    if FNumericType = ntPercentage then
      FValue := X / 100
    Else
      FValue := X;
    FormatText;
    Inherited;
    End
  Else
    Begin
    SelectAll;
    SetFocus;
    End;
End;

Procedure TIsamCustomNumEdit.FormatText;
Var
  X : extended;
  Multiplier : real;
Begin
  {round the number appropriately}
  Try
    Multiplier := Power ( 10, Decimals );
    if FNumericType = ntPercentage then
      X := FValue * 100
    Else
      X := FValue;
    if UseRounding then
      X := round ( X * Multiplier ) / Multiplier
    Else
      X := trunc ( X * Multiplier ) / Multiplier;
  Except
    on ERangeError do
      X := FValue; {will cause rounding in the FloatToStr function}
  End;

  {format the number}
  Case FNumericType of
    ntCurrency   : Text := FloatToStrF ( X, ffCurrency, FDigits, FDecimals);
    ntPercentage : Text := FloatToStrF ( X, ffFixed, FDigits, FDecimals) + '%';
    ntGeneral    : With Masks do
                     Text := FormatFloat( PositiveMask+';'+NegativeMask+';'+ZeroMask, X);
  End;
End;

Procedure TIsamCustomNumEdit.MaskChanged ( Sender : TObject );
Begin
  FormatText;
End;

Procedure TIsamCustomNumEdit.UnFormatText;
Begin
  Text := StripChars ( Text, [ '0'..'9', DecimalSeparator, ThousandSeparator ] );
  if Value < 0 then
    Text := '-' + Text;
End;

{--- TIsamDateButton ---}
Procedure TIsamDateButton.Click;
Var
  EditParent: TIsamDateEdit;
Begin
  EditParent := TIsamDateEdit( Parent );
  ItabCalPop := TItabCalPop.Create( editParent );
  ITabCalPop.ShowModal;
  ITabCalPop.Free;
  Inherited Click;
  EditParent.SetFocus;
  EditParent.Modified := True;
End;

Constructor TIsamDateEdit.Create( AOwner: TComponent );
Var
  ini: TIniFile;
Begin
  Inherited Create( AOwner );
{ Get international time seperator }
  FDate := 0.0;
  FButton := TIsamDateButton.Create( self );
  FButton.Visible := True;
  FButton.Parent := self;
  FButton.TabStop:= False;
  FButton.Glyph.Handle := LoadBitmap( hInstance, 'ITABCALPOP' );
  ControlStyle := ControlStyle - [csSetCaption];
End;

Procedure TISAMDateEdit.DataChange(Sender: TObject);
Var I   : Byte;
    L   : Longint;
    Ref : Longint;
    B   : Boolean;
Begin
  FCtrlLink.NoEdit := True;
  if FCtrlLink.TableActive then
  Begin
    Text := DataSource.DataSet.StringGetN(FCtrlLink.FieldNumber);
  End
  Else Begin
    MaxLength := 0;
    if csDesigning in ComponentState then
    Text := Name Else
    Text := '';
  End;
  Modified := False;
  FCtrlLink.NoEdit := False;
End;

Procedure TIsamDateEdit.WMSize( Var Message: TWMSize );
Begin
  {$IFDEF VER90}
  FButton.Height := Height - 2;
  FButton.Width := Height - 2;
  FButton.Left:= Width - Height - 1;
  {$ELSE}
  FButton.Height:= Height;
  FButton.Width:= Height;
  FButton.Left := Width - Height;
  {$ENDIF}
  FButton.Refresh;
End;

Procedure TIsamDateEdit.KeyPress( Var Key: Char );
Var
  d: TDateTime;
Begin
  if upstr(Key) = 'H' then
  Begin
    Key := #0;
    D := Now;
    Date := Now;
    Text := DatetoStr( D);
    ModiFied := True;
    Exit;
  End;
  if ( ( Key < '0' ) or ( Key > '9' ) )  and ( Key <> #8 )
  and (Key <> #13) and (Key <> '.') and (Key <> '-') and (Key <> '/') then Key := #0
  Else Inherited KeyPress( Key );
End;


Destructor TIsamDateEdit.Destroy;
Begin
  FButton := Nil;
  Inherited Destroy;
End;

Procedure TIsamDateEdit.CreateParams( Var Params: TCreateParams );
Begin
  Inherited CreateParams( Params );
  Params.Style := Params.Style or WS_CLIPCHILDREN;
End;

Function TIsamDateEdit.GetDate: TDateTime;
Begin
  GetDate := FDate;
End;

Procedure TIsamDateEdit.SetDate( dtArg: TDateTime );
Begin
  if (FDate <> dtArg) or (Text = '') then Begin
    FDate := dtArg;
    Modified := True;
    if FDate = 0 then Text := ''
    Else Begin
      Text := DateToStr(FDate);
    End;
  End;
End;


End.


