unit ITab_reg;
{copyright 1995,1996,1997 by Stellberg Computer GmbH}
interface

uses Classes, DsgnIntf;

procedure Register;

implementation

{$Ifdef Ver90}
{$R ITab_REG.D32}
{$Else}
{$R ITab_REG.D16}
{$Endif}


uses Controls, WinTypes, WinProcs, Forms, SysUtils, LibConst, TypInfo,
     ITable, ITblCtrl, Dialogs, ITabBrow, ITab_Def, ITabDat,
     ITabEdit, ITabExcl, ITabDbas, Db, ITabExpt, ExptIntf;

{ TIStringProperty }

type
  TIStringProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValueList(List: TStrings); virtual; abstract;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

function TIStringProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paMultiSelect];
end;

procedure TIStringProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
  Values: TStringList;
begin
  Values := TStringList.Create;
  try
    GetValueList(Values);
    for I := 0 to Values.Count - 1 do Proc(Values[I]);
  finally
    Values.Free;
  end;
end;

{ TTableNameProperty }

type
  TITableNameProperty = class(TIStringProperty)
  public
    procedure GetValueList(List: TStrings); override;
  end;

procedure TITableNameProperty.GetValueList(List: TStrings);
begin
   (GetComponent(0) as TITable).GetTableNames(List);
end;

{ TIndexProperty }

type
  TIndexProperty = class(TIStringProperty)
  public
    procedure GetValueList(List: TStrings); override;
  end;

procedure TIndexProperty.GetValueList(List: TStrings);
begin
  (GetComponent(0) as TITable).GetIndexList(List);
end;

{ TDataFieldProperty }

type
  TDataFieldProperty = class(TIStringProperty)
  public
    procedure GetValueList(List: TStrings); override;
  end;

procedure TDataFieldProperty.GetValueList(List: TStrings);
var
  Instance: TComponent;
  PropInfo: PPropInfo;
  DataSource: TIsamDatasource;
begin
  Instance := GetComponent(0);
  PropInfo := TypInfo.GetPropInfo(Instance.ClassInfo, 'DataSource');
  if (PropInfo <> nil) and (PropInfo^.PropType^.Kind = tkClass) then
  begin
    DataSource := TObject(GetOrdProp(Instance, PropInfo)) as TIsamDatasource;
    if (DataSource <> nil) then begin
      if DataSource.DataSet <> NIL then
      DataSource.DataSet.GetFieldNames(List)
    end
  end
end;

{ TPickDataFieldProperty }

type
  TPickDataFieldProperty = class(TIStringProperty)
  public
    procedure GetValueList(List: TStrings); override;
  end;

procedure TPickDataFieldProperty.GetValueList(List: TStrings);
var
  Instance: TComponent;
  PropInfo: PPropInfo;
  DataSource: TITable;
begin
  Instance := GetComponent(0);
  PropInfo := TypInfo.GetPropInfo(Instance.ClassInfo, 'PickIsamTable');
  if (PropInfo <> nil) and (PropInfo^.PropType^.Kind = tkClass) then
  begin
    DataSource := TObject(GetOrdProp(Instance, PropInfo)) as TITable;
    if (DataSource <> nil) then
      DataSource.GetFieldNames(List);
  end;
end;

{ TStructureProperty }
type
  TStructureProperty = class(TClassProperty)
  public
    function  GetAttributes: TPropertyAttributes; override;
    function GetValue: String; override;
    procedure Edit; override;
  end;

function TStructureProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog] - [paSubProperties];
end;

function TStructureProperty.GetValue: String;
begin
   Result := (GetComponent(0) as TITable).Structure;
end;

procedure TStructureProperty.Edit;
var
  Frm : TITabDefDlg;
  Str,TblNam : ShortString;
begin
  TblNam:= (GetComponent(0) as TITable).TableName;
  if TblNam = '' then MessageDlg('no TableName',mtError,[mbOk], 0)
  else begin
    Frm := TITabDefDlg.Create(Application);
    try
      Str:= (GetComponent(0) as TITable).Structure;
      Frm.TblName:= TblNam;
      if (Str <> '') and (FileExists(__Laufwerk + Str)) then Frm.Lade(__Laufwerk+Str);
      if Frm.ShowModal = idOK then begin
        (GetComponent(0) as TITable).Structure:= Frm.DateiName;
        Designer.Modified;
      end;
    finally
      Frm.Free;
    end;
  end;
end;

{ TPathProperty }
type
  TPathProperty = class(TClassProperty)
  public
    function  GetAttributes: TPropertyAttributes; override;
    function GetValue: String; override;
    procedure Edit; override;
  end;

function TPathProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog] - [paSubProperties];
end;

function TPathProperty.GetValue: String;
begin
   Result := (GetComponent(0) as TITable).TablePath;
end;

procedure TPathProperty.Edit;
begin
  LwSetup := TLw_Setup.Create(Application);
  try
    LwSetup.ShowModal;
    (GetComponent(0) as TITable).SetTablePath(__Laufwerk);
    Designer.Modified;
 finally
    LwSetup.Free;
  end;
end;

{ TLCFieldProperty }
type
  TLCFieldProperty = class(TClassProperty)
  public
    function  GetAttributes: TPropertyAttributes; override;
    function GetValue: String; override;
    procedure Edit; override;
  end;

function TLCFieldProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog] - [paSubProperties];
end;

function TLCFieldProperty.GetValue: String;
begin
   Result := 'GridColumns';
end;

procedure TLCFieldProperty.Edit;
begin
end;

{ TIDatabaseNameProperty }

type
  TIDatabaseNameProperty = class(TIStringProperty)
  public
    procedure GetValueList(List: TStrings); override;
  end;

procedure TIDatabaseNameProperty.GetValueList(List: TStrings);
begin
  Session.GetDatabaseNames(List);
end;

procedure Register;
begin
  RegisterComponents('I-Table',[TITable, TIsamDatasource,
                     TITabBrowser,TItabNavigator,TIsamHeader, TIsamText,TIsamEdit,
                     TIsamDateEdit,TIsamStrEdit,TIsamNumEdit,TIsamKeyCtrl,
                     TITabCheckBox,
                     ExcelExportCtrl,DBaseExportCtrl]);
  RegisterPropertyEditor(TypeInfo(Shortstring), TITable,
                         'TableName', TITableNameProperty);
  RegisterPropertyEditor(TypeInfo(ShortString), TComponent, 'IsamField',
                         TDataFieldProperty);
  RegisterPropertyEditor(TypeInfo(ShortString), TITable, 'KeyIndex', TIndexProperty);
  RegisterPropertyEditor(TypeInfo(ShortString), TITable, 'Structure', TStructureProperty);
  RegisterPropertyEditor(TypeInfo(ShortString), TITable, 'TablePath', TPathProperty);
  RegisterLibraryExpert(TItabDialogExpert.Create);
end;


end.

